@@ -1,48 +1,61 @@
-from UserDict import DictMixin
+
+# This code was inspired by https://github.com/andrewdyates/xmldsig
+# and includes https://github.com/andrewdyates/rsa_x509_pem with
+# permission from the author.
 
 __author__ = 'leifj'
 
 import os
-import xmlsec.rsa_x509_pem as rsa_x509_pem
-import lxml.etree as etree
+from . import rsa_x509_pem
+from lxml import etree as etree
 import logging
 import base64
 import hashlib
 import copy
-import xmlsec.int_to_bytes as itb
+from . import int_to_bytes as itb
 from lxml.builder import ElementMaker
 from xmlsec.exceptions import XMLSigException
-import re
-import htmlentitydefs
+from UserDict import DictMixin
+from xmlsec import constants
+from xmlsec.utils import parse_xml, pem2b64, unescape_xml_entities, delete_elt, root_elt, b64d, b64e
+import pyconfig
 
 NS = {'ds': 'http://www.w3.org/2000/09/xmldsig#'}
 NSDefault = {None: 'http://www.w3.org/2000/09/xmldsig#'}
 DS = ElementMaker(namespace=NS['ds'], nsmap=NSDefault)
 
-# Enable this to get various parts written to files in /tmp. Not for production!
-_DEBUG_WRITE_TO_FILES = False
 
-# ASN.1 BER SHA1 algorithm designator prefixes (RFC3447)
-ASN1_BER_ALG_DESIGNATOR_PREFIX = {
-    # disabled 'md2': '\x30\x20\x30\x0c\x06\x08\x2a\x86\x48\x86\xf7\x0d\x02\x02\x05\x00\x04\x10',
-    # disabled 'md5': '\x30\x20\x30\x0c\x06\x08\x2a\x86\x48\x86\xf7\x0d\x02\x05\x05\x00\x04\x10',
-    'sha1': '\x30\x21\x30\x09\x06\x05\x2b\x0e\x03\x02\x1a\x05\x00\x04\x14',
-    'sha256': '\x30\x31\x30\x0d\x06\x09\x60\x86\x48\x01\x65\x03\x04\x02\x01\x05\x00\x04\x20',
-    'sha384': '\x30\x41\x30\x0d\x06\x09\x60\x86\x48\x01\x65\x03\x04\x02\x02\x05\x00\x04\x30',
-    'sha512': '\x30\x51\x30\x0d\x06\x09\x60\x86\x48\x01\x65\x03\x04\x02\x03\x05\x00\x04\x40',
-}
+class Config(object):
+    """
+    This class holds a set of configuration parameters (using pyconfig) for pyXMLSecurity:
 
-TRANSFORM_ENVELOPED_SIGNATURE = 'http://www.w3.org/2000/09/xmldsig#enveloped-signature'
-TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS = 'http://www.w3.org/2001/10/xml-exc-c14n#WithComments'
-TRANSFORM_C14N_EXCLUSIVE = 'http://www.w3.org/2001/10/xml-exc-c14n#'
-TRANSFORM_C14N_INCLUSIVE = 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315'
+    :param default_signature_alg: The URI of the default signature algorithm (RSA_SHA1 by default)
+    :param default_digest_alg: The URI of the default digest algorithm (SHA1 by default)
+    :param default_c14n_alg: The URI of the default c14n algorithm (c14n exclusive by default)
+    :param debug_write_to_files: Set to True to dump certain XML traces to /tmp. Danger! Not for production!
+    :param same_document_is_root: Set to True to treat implicit null same-document-references as reference to the whole document.
+    :param id_attributes: A list of attributes to be used as 'id's. By default set to ['ID','id']
+    :param c14n_strip_ws: Set to True to strip whitespaces in c14n. Only use if you have very special needs.
 
-ALGORITHM_DIGEST_SHA1 = "http://www.w3.org/2000/09/xmldsig#sha1"
-ALGORITHM_SIGNATURE_RSA_SHA1 = "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
+    Refer to the pyconfig documentation for information on how to override these in your own project.
+    """
+    default_signature_alg = pyconfig.setting("xmlsec.default_signature_alg", constants.ALGORITHM_SIGNATURE_RSA_SHA1)
+    default_digest_alg = pyconfig.setting("xmlsec.default_digest_alg", constants.ALGORITHM_DIGEST_SHA1)
+    default_c14n_alg = pyconfig.setting("xmlsec.default_c14n_alg", constants.TRANSFORM_C14N_INCLUSIVE)
+    debug_write_to_files = pyconfig.setting("xmlsec.config.debug_write_to_files", False)
+    same_document_is_root = pyconfig.setting("xmlsec.same_document_is_root", False)
+    id_attributes = pyconfig.setting("xmlsec.id_attributes", ['ID', 'id'])
+    c14n_strip_ws = pyconfig.setting("xmlsec.c14n_strip_ws", False)
 
-# This code was inspired by https://github.com/andrewdyates/xmldsig
-# and includes https://github.com/andrewdyates/rsa_x509_pem with
-# permission from the author.
+
+config = Config()
+
+
+def _implicit_same_document(t, sig):
+    if config.same_document_is_root:
+        return root_elt(copy.deepcopy(t))
+    else:
+        return copy.deepcopy(sig.getparent())
 
 
 class CertDict(DictMixin):
@@ -144,7 +157,7 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
                 data = c.read()
             source = 'file'
         elif private and keyspec.startswith("pkcs11://"):
-            import xmlsec.pk11
+            from xmlsec import pk11
 
             key_f_private, data = pk11.signer(keyspec)
             logging.debug("Using pkcs11 signing key: %s" % key_f_private)
@@ -182,35 +195,6 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
     return res
 
 
-def _root(t):
-    if hasattr(t, 'getroot') and hasattr(t.getroot, '__call__'):
-        return t.getroot()
-    else:
-        return t
-
-
-def number_of_bits(num):
-    """
-    Return the number of bits required to represent num.
-
-    In python >= 2.7, there is num.bit_length().
-
-    NOTE: This function appears unused, so it might go away.
-    """
-    assert num >= 0
-    # this is much faster than you would think, AND it is easy to read ;)
-    return len(bin(num)) - 2
-
-
-b64d = lambda s: s.decode('base64')
-
-
-def b64e(s):
-    if type(s) in (int, long):
-        s = itb.int_to_bytes(s)
-    return s.encode('base64').replace('\n', '')
-
-
 def _signed_value(data, key_size, do_pad, hash_alg):  # TODO Do proper asn1 CMS
     """Return unencrypted rsa-sha1 signature value `padded_digest` from `data`.
 
@@ -225,7 +209,7 @@ def _signed_value(data, key_size, do_pad, hash_alg):  # TODO Do proper asn1 CMS
       str: rsa-sha1 signature value of `data`
     """
 
-    prefix = ASN1_BER_ALG_DESIGNATOR_PREFIX.get(hash_alg)
+    prefix = constants.ASN1_BER_ALG_DESIGNATOR_PREFIX.get(hash_alg)
     if not prefix:
         raise XMLSigException("Unknown hash algorithm %s" % hash_alg)
     asn_digest = prefix + data
@@ -249,13 +233,14 @@ def _digest(data, hash_alg):
     :returns: Base64 string
     """
     h = getattr(hashlib, hash_alg)()
+    logging.debug(h)
     h.update(data)
-    digest = b64e(h.digest())
+    digest = h.digest().encode("base64").rstrip("\n") #b64e(h.digest())
     return digest
 
 
 def _get_by_id(t, id_v):
-    for id_a in _id_attributes:
+    for id_a in config.id_attributes:
         logging.debug("Looking for #%s using id attribute '%s'" % (id_v, id_a))
         elts = t.xpath("//*[@%s='%s']" % (id_a, id_v))
         if elts is not None and len(elts) > 0:
@@ -276,27 +261,26 @@ def _alg(elt):
 
 
 def _remove_child_comments(t):
-    #root = _root(t)
+    #root = root_elt(t)
     for c in t.iter():
         if c.tag is etree.Comment or c.tag is etree.PI:
-            _delete_elt(c)
+            delete_elt(c)
     return t
 
 
-def _process_references(t, sig=None, return_verified=True):
+def _process_references(t, sig, return_verified=True, sig_path=".//{%s}Signature" % NS['ds']):
     """
     :returns: hash algorithm as string
     """
-    if sig is None:
-        sig = t.find(".//{%s}Signature" % NS['ds'])
-    hash_alg = None
+
+    verified_objects = []
     for ref in sig.findall(".//{%s}Reference" % NS['ds']):
         obj = None
-        verified = None
+        hash_alg = None
         uri = ref.get('URI', None)
         if uri is None or uri == '#' or uri == '':
-            ct = _remove_child_comments(_root(copy.deepcopy(t)))
-            obj = _root(ct)
+            ct = _remove_child_comments(_implicit_same_document(t, sig))
+            obj = root_elt(ct)
         elif uri.startswith('#'):
             ct = copy.deepcopy(t)
             obj = _remove_child_comments(_get_by_id(ct, uri[1:]))
@@ -307,116 +291,71 @@ def _process_references(t, sig=None, return_verified=True):
             raise XMLSigException("Unable to dereference Reference URI='%s'" % uri)
 
         if return_verified:
-            verified = copy.deepcopy(obj)
+            verified_objects.append(copy.deepcopy(obj))
+
+        if config.debug_write_to_files:
+            with open("/tmp/foo-pre-transform.xml", "w") as fd:
+                fd.write(etree.tostring(obj))
 
         for tr in ref.findall(".//{%s}Transform" % NS['ds']):
             logging.debug("transform: %s" % _alg(tr))
-            obj = _transform(_alg(tr), obj, tr)
+            obj = _transform(_alg(tr), obj, tr=tr, sig_path=sig_path)
 
         if not isinstance(obj, basestring):
-            obj = _unescape(etree.tostring(obj).decode("utf8", 'replace')).encode("utf8").strip()
+            if config.debug_write_to_files:
+                with open("/tmp/foo-pre-serialize.xml", "w") as fd:
+                    fd.write(etree.tostring(obj))
+            obj = _transform(constants.TRANSFORM_C14N_INCLUSIVE, obj)
 
-        if _DEBUG_WRITE_TO_FILES:
+        if config.debug_write_to_files:
             with open("/tmp/foo-obj.xml", "w") as fd:
                 fd.write(obj)
 
         dm = ref.find(".//{%s}DigestMethod" % NS['ds'])
         if dm is None:
             raise XMLSigException("Unable to find DigestMethod")
-        this_hash_alg = (_alg(dm).split("#"))[1]
-        logging.debug("using hash algorithm %s" % this_hash_alg)
-        hash_alg = hash_alg or this_hash_alg
-        if this_hash_alg != hash_alg:
-            raise XMLSigException("Unable to handle more than one hash algorithm (%s != %s)"
-                                  % (this_hash_alg, hash_alg))
-        digest = _digest(obj, this_hash_alg)
-        logging.debug("digest for %s: %s" % (uri, digest))
+        hash_alg = (_alg(dm).split("#"))[1]
+        logging.debug("using hash algorithm %s" % hash_alg)
+        digest = _digest(obj, hash_alg)
+        logging.debug("using digest %s (%s) for ref %s" % (digest, hash_alg, uri))
         dv = ref.find(".//{%s}DigestValue" % NS['ds'])
         logging.debug(etree.tostring(dv))
         dv.text = digest
 
     if return_verified:
-        return hash_alg, verified
+        return verified_objects
     else:
-        return hash_alg
-
-##
-# Removes HTML or XML character references and entities from a text string.
-#
-# @param text The HTML (or XML) source text.
-# @return The plain text, as a Unicode string, if necessary.
+        return None
 
 
-def _unescape(text):
-    def fixup(m):
-        text = m.group(0)
-        if text[:2] == "&#":
-            # character reference
-            try:
-                if text[:3] == "&#x":
-                    return unichr(int(text[3:-1], 16))
-                else:
-                    return unichr(int(text[2:-1]))
-            except ValueError:
-                pass
-        else:
-            # named entity
-            try:
-                if not text in ('&amp;', '&lt;', '&gt;'):
-                    text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
-            except KeyError:
-                pass
-        return text  # leave as is
-
-    return re.sub("&#?\w+;", fixup, text)
-
-
-def _delete_elt(elt):
-    if elt.getparent() is None:
-        raise XMLSigException("Cannot delete root")
-    if elt.tail is not None:
-        logging.debug("tail: '%s'" % elt.tail)
-        p = elt.getprevious()
-        if p is not None:
-            logging.debug("adding tail to previous")
-            if p.tail is None:
-                p.tail = ''
-            p.tail += elt.tail
-        else:
-            logging.debug("adding tail to parent")
-            up = elt.getparent()
-            if up is None:
-                raise XMLSigException("Signature has no parent")
-            if up.text is None:
-                up.text = ''
-            up.text += elt.tail
-    elt.getparent().remove(elt)
-
-
-def _enveloped_signature(t):
-    sig = t.find('.//{http://www.w3.org/2000/09/xmldsig#}Signature')
-    _delete_elt(sig)
-    if _DEBUG_WRITE_TO_FILES:
+def _enveloped_signature(t, sig_path=".//{%s}Signature" % NS['ds']):
+    sig = t.find(sig_path)
+    if sig is not None:
+        delete_elt(sig)
+    if config.debug_write_to_files:
         with open("/tmp/foo-env.xml", "w") as fd:
             fd.write(etree.tostring(t))
     return t
 
-def _c14n(t, exclusive, with_comments, inclusive_prefix_list=None):
+
+def _c14n(t, exclusive, with_comments, inclusive_prefix_list=None, schema=None):
     """
     Perform XML canonicalization (c14n) on an lxml.etree.
 
-    NOTE: The c14n done here is missing whitespace removal. The whitespace has to
-    be removed at parse time. One way to do that is to use xmlsec.parse_xml().
-
     :param t: XML as lxml.etree
     :param exclusive: boolean
     :param with_comments: boolean, keep comments or not
     :param inclusive_prefix_list: List of namespaces to include (?)
     :returns: XML as string (utf8)
     """
-    cxml = etree.tostring(t, method="c14n", exclusive=exclusive, with_comments=with_comments,
-                          inclusive_ns_prefixes=inclusive_prefix_list)
-    u = _unescape(cxml.decode("utf8", 'replace')).encode("utf8").strip()
+    xml_str = etree.tostring(t)
+    doc = parse_xml(xml_str, remove_whitespace=config.c14n_strip_ws, remove_comments=not with_comments, schema=schema)
+    buf = etree.tostring(doc,
+                         method='c14n',
+                         exclusive=exclusive,
+                         with_comments=with_comments,
+                         inclusive_ns_prefixes=inclusive_prefix_list)
+    u = unescape_xml_entities(buf.decode("utf8", 'replace')).encode("utf8").strip()
     if u[0] != '<':
         raise XMLSigException("C14N buffer doesn't start with '<'")
     if u[-1] != '>':
@@ -424,65 +363,37 @@ def _c14n(t, exclusive, with_comments, inclusive_prefix_list=None):
     return u
 
 
-def _transform(uri, t, tr=None):
-    if uri == TRANSFORM_ENVELOPED_SIGNATURE:
-        return _enveloped_signature(t)
+def _transform(uri, t, tr=None, schema=None, sig_path=".//{%s}Signature" % NS['ds']):
+    if uri == constants.TRANSFORM_ENVELOPED_SIGNATURE:
+        return _enveloped_signature(t, sig_path)
 
-    if uri == TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS:
+    if uri == constants.TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS:
         nslist = None
         if tr is not None:
             elt = tr.find(".//{%s}InclusiveNamespaces" % 'http://www.w3.org/2001/10/xml-exc-c14n#')
             if elt is not None:
                 nslist = elt.get('PrefixList', '').split()
-        return _c14n(t, exclusive=True, with_comments=True, inclusive_prefix_list=nslist)
+        return _c14n(t, exclusive=True, with_comments=True, inclusive_prefix_list=nslist, schema=schema)
 
-    if uri == TRANSFORM_C14N_EXCLUSIVE:
+    if uri == constants.TRANSFORM_C14N_EXCLUSIVE:
         nslist = None
         if tr is not None:
             elt = tr.find(".//{%s}InclusiveNamespaces" % 'http://www.w3.org/2001/10/xml-exc-c14n#')
             if elt is not None:
                 nslist = elt.get('PrefixList', '').split()
-        return _c14n(t, exclusive=True, with_comments=False, inclusive_prefix_list=nslist)
+        return _c14n(t, exclusive=True, with_comments=False, inclusive_prefix_list=nslist, schema=schema)
 
-    if uri == TRANSFORM_C14N_INCLUSIVE:
-        return _c14n(t, exclusive=False, with_comments=False)
+    if uri == constants.TRANSFORM_C14N_INCLUSIVE:
+        return _c14n(t, exclusive=False, with_comments=False, schema=schema)
 
     raise XMLSigException("unknown or unimplemented transform %s" % uri)
 
 
-_id_attributes = ['ID', 'id']
-
-
 def setID(ids):
-    _id_attributes = ids
+    constants.id_attributes = ids
 
 
-def pem2b64(pem):
-    return '\n'.join(pem.strip().split('\n')[1:-1])
-
-
-def b642pem(data):
-    x = data
-    r = "-----BEGIN CERTIFICATE-----\n"
-    while len(x) > 64:
-        r += x[0:64]
-        r += "\n"
-        x = x[64:]
-    r += x
-    r += "\n"
-    r += "-----END CERTIFICATE-----"
-    return r
-
-
-def pem2cert(pem):
-    return rsa_x509_pem.parse(pem)
-
-
-def b642cert(data):
-    return rsa_x509_pem.parse(b642pem(data))
-
-
-def _verify(t, keyspec):
+def _verify(t, keyspec, sig_path=".//{%s}Signature" % NS['ds']):
     """
     Verify the signature(s) in an XML document.
 
@@ -492,15 +403,15 @@ def _verify(t, keyspec):
     :param keyspec: X.509 cert filename, string with fingerprint or X.509 cert as string
     :returns: True if signature(s) validated, False if there were no signatures
     """
-    if _DEBUG_WRITE_TO_FILES:
+    if config.debug_write_to_files:
         with open("/tmp/foo-sig.xml", "w") as fd:
-            fd.write(etree.tostring(_root(t)))
+            fd.write(etree.tostring(root_elt(t)))
 
     # Load and parse certificate, unless keyspec is a fingerprint.
     cert = _load_keyspec(keyspec)
 
     validated = []
-    for sig in t.findall(".//{%s}Signature" % NS['ds']):
+    for sig in t.findall(sig_path):
         sv = sig.findtext(".//{%s}SignatureValue" % NS['ds'])
         if sv is None:
             raise XMLSigException("No SignatureValue")
@@ -525,27 +436,28 @@ def _verify(t, keyspec):
         if this_cert is None:
             raise XMLSigException("Could not find certificate to validate signature")
 
-        hash_alg, ref = _process_references(t, sig)
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
         cm_alg = _cm_alg(si)
-        b_digest = _create_signature_digest(si, cm_alg, hash_alg)
+        digest_alg = _sig_digest(si)
 
-        actual = _signed_value(b_digest, this_keysize, True, hash_alg)
+        validated_objects = _process_references(t, sig, sig_path)
+        b_digest = _create_signature_digest(si, cm_alg, digest_alg)
+        actual = _signed_value(b_digest, this_keysize, True, digest_alg)
         expected = this_f_public(b64d(sv))
 
         if expected != actual:
             raise XMLSigException("Signature validation failed")
-        validated.append(ref)
+        validated.extend(validated_objects)
 
     return validated
 
 
-def verify(t, keyspec):
-    return len(_verify(t, keyspec)) > 0
+def verify(t, keyspec, sig_path=".//{%s}Signature" % NS['ds']):
+    return len(_verify(t, keyspec, sig_path)) > 0
 
 
-def verified(t, keyspec):
-    return _verify(t, keyspec)
+def verified(t, keyspec, sig_path=".//{%s}Signature" % NS['ds']):
+    return _verify(t, keyspec, sig_path)
 
 
 ## TODO - support transforms with arguments
@@ -554,12 +466,16 @@ def _signed_info_transforms(transforms):
     return DS.Transforms(*ts)
 
 
-# standard enveloped rsa-sha1 signature
-def _enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri):
+# standard enveloped signature
+def _enveloped_signature_template(c14n_method,
+                                  digest_alg,
+                                  transforms,
+                                  reference_uri,
+                                  signature_alg):
     return DS.Signature(
         DS.SignedInfo(
             DS.CanonicalizationMethod(Algorithm=c14n_method),
-            DS.SignatureMethod(Algorithm=ALGORITHM_SIGNATURE_RSA_SHA1),
+            DS.SignatureMethod(Algorithm=signature_alg),
             DS.Reference(
                 _signed_info_transforms(transforms),
                 DS.DigestMethod(Algorithm=digest_alg),
@@ -571,20 +487,24 @@ def _enveloped_signature_template(c14n_method, digest_alg, transforms, reference
 
 
 def add_enveloped_signature(t,
-                            c14n_method=TRANSFORM_C14N_INCLUSIVE,
-                            digest_alg=ALGORITHM_DIGEST_SHA1,
+                            c14n_method=config.default_c14n_alg,
+                            digest_alg=config.default_digest_alg,
+                            signature_alg=config.default_signature_alg,
                             transforms=None,
                             reference_uri='',
                             pos=0):
     if transforms is None:
-        transforms = (TRANSFORM_ENVELOPED_SIGNATURE, TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS)
+        transforms = (constants.TRANSFORM_ENVELOPED_SIGNATURE,
+                      constants.TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS)
+
+    tmpl = _enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri, signature_alg)
     if pos == -1:
-        _root(t).append(_enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri))
+        root_elt(t).append(tmpl)
     else:
-        _root(t).insert(pos, _enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri))
+        root_elt(t).insert(pos, tmpl)
 
 
-def sign(t, key_spec, cert_spec=None, reference_uri=''):
+def sign(t, key_spec, cert_spec=None, reference_uri='', sig_path=".//{%s}Signature" % NS['ds']):
     """
     Sign an XML document. This means to 'complete' all Signature elements in the XML.
 
@@ -615,22 +535,25 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
                               % (public['keysize'], private['keysize']))
     logging.debug("Using %s bit key" % (private['keysize']))
 
-    if t.find(".//{%s}Signature" % NS['ds']) is None:
+    if t.find(sig_path) is None:
         add_enveloped_signature(t, reference_uri=reference_uri)
 
-    if _DEBUG_WRITE_TO_FILES:
+    if config.debug_write_to_files:
         with open("/tmp/sig-ref.xml", "w") as fd:
-            fd.write(etree.tostring(_root(t)))
+            fd.write(etree.tostring(root_elt(t)))
 
-    for sig in t.findall(".//{%s}Signature" % NS['ds']):
-        hash_alg = _process_references(t, sig, return_verified=False)
-        # XXX create signature reference duplicates/overlaps process references unless a c14 is part of transforms
+    for sig in t.findall(sig_path):
+        logging.debug("processing sig template: %s" % etree.tostring(sig))
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
         cm_alg = _cm_alg(si)
-        b_digest = _create_signature_digest(si, cm_alg, hash_alg)
+        digest_alg = _sig_digest(si)
+
+        _process_references(t, sig, return_verified=False, sig_path=sig_path)
+        # XXX create signature reference duplicates/overlaps process references unless a c14 is part of transforms
+        b_digest = _create_signature_digest(si, cm_alg, digest_alg)
 
         # sign hash digest and insert it into the XML
-        tbs = _signed_value(b_digest, private['keysize'], do_padding, hash_alg)
+        tbs = _signed_value(b_digest, private['keysize'], do_padding, digest_alg)
         signed = private['f_private'](tbs)
         signature = b64e(signed)
         logging.debug("SignatureValue: %s" % signature)
@@ -652,10 +575,23 @@ def _cm_alg(si):
     return cm_alg
 
 
+def _sig_alg(si):
+    sm = si.find(".//{%s}SignatureMethod" % NS['ds'])
+    sig_alg = _alg(sm)
+    if sm is None or sig_alg is None:
+        raise XMLSigException("No SignatureMethod")
+    return (sig_alg.split("#"))[1]
+
+
+def _sig_digest(si):
+    return (_sig_alg(si).split("-"))[1]
+
+
 def _create_signature_digest(si, cm_alg, hash_alg):
     """
     :param hash_alg: string such as 'sha1'
     """
+    logging.debug("transform %s on %s" % (cm_alg, etree.tostring(si)))
     sic = _transform(cm_alg, si)
     logging.debug("SignedInfo C14N: %s" % sic)
     digest = _digest(sic, hash_alg)
@@ -663,13 +599,3 @@ def _create_signature_digest(si, cm_alg, hash_alg):
     return b64d(digest)
 
 
-def parse_xml(data, remove_whitespace=True, remove_comments=True):
-    """
-    Parse XML data into an lxml.etree and remove whitespace in the process.
-
-    :param data: XML as string
-    :param remove_whitespace: boolean
-    :returns: XML as lxml.etree
-    """
-    parser = etree.XMLParser(remove_blank_text=remove_whitespace, remove_comments=remove_comments)
-    return etree.XML(data, parser)
